# -*- coding=utf-8 -*-

from db.src.db_helper_mssql import *
from db.src.db_helper_pg import *

from helpers.src.app_redis_helper import *
from helpers.src.crypto_helper import *
from helpers.src.download_helper import *
from helpers.src.flask_helper import *
from helpers.src.io_helper import *
from helpers.src.redis_helper import *
from helpers.src.requests_helper import *
from helpers.src.utilities_helper import *




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from Crypto.Cipher import AES

import binascii


class PKCS7Encoder(object):

    """
    Python implementation of PKCS #7 padding.
    RFC 2315: PKCS#7 page 21
    Some content-encryption algorithms assume the
    input length is a multiple of k octets, where k > 1, and
    let the application define a method for handling inputs
    whose lengths are not a multiple of k octets. For such
    algorithms, the method shall be to pad the input at the
    trailing end with k - (l mod k) octets all having value k -
    (l mod k), where l is the length of the input. In other
    words, the input is padded at the trailing end with one of
    the following strings:
            01 -- if l mod k = k-1
            02 02 -- if l mod k = k-2
                        .
                        .
                        .
        k k ... k k -- if l mod k = 0
    The padding can be removed unambiguously since all input is
    padded and no padding string is a suffix of another. This
    padding method is well-defined if and only if k < 256;
    methods for larger k are an open issue for further study.
    """

    # Original Source: http://japrogbits.blogspot.com/2011/02/using-encrypted-data-between-python-and.html

    # Updated to Python 3.
    #   Despite some complaints of Python 3, using it correctly simplified the
    #   original greatly.

    # @param bytestring    The padded bytestring for which the padding is to be removed.
    # @param k             The padding block size.
    # @exception ValueError Raised when the input padding is missing or corrupt.
    # @return bytestring    Original unpadded bytestring.
    @staticmethod
    def decode(bytestring, k=16):
        """
        Remove the PKCS#7 padding from a text bytestring.
        """

        val = bytestring[-1]
        if val > k:
            raise ValueError('Input is not padded or padding is corrupt')
        l = len(bytestring) - val
        return bytestring[:l]

    # @param bytestring    The text to encode.
    # @param k             The padding block size.
    # @return bytestring    The padded bytestring.
    @staticmethod
    def encode(bytestring, k=16):
        """
        Pad an input bytestring according to PKCS#7

        """
        l = len(bytestring.encode('utf-8'))
        val = k - (l % k)
        # return bytestring + binascii.unhexlify([val] * val)
        # p = bytearray([val] * val)
        # return bytestring + p.decode('utf-8')
        return bytes(bytestring, 'utf-8') + bytes([val] * val)


# Tests
# =====
# import unittest

# class Pkcs7Test(unittest.TestCase):
#     def test_pkcs7(self):
#         # Not a unit at all, but whatever.
#         import pkcs7
#         import os
#         import random

#         def test(text):
#             a = pkcs7.encode(text)
#             b = pkcs7.encode(text)
#             assert len(a) % 4 == 0
#             assert len(b) % 4 == 0
#             assert a == b

#         for i in range(1000):
#             text = os.urandom( int(random.random()*10000) )
#             test(text)



# -*- coding=utf-8 -*-

from enum import Enum
import sys
import psycopg2
from psycopg2.extras import DictCursor, RealDictCursor

from db.src.db_helper_base import BaseDBHelper


class DBOperatorEnum(Enum):
    EQUAL_TO = '='
    NOT_EQUAL_TO = '!='
    LIKE = 'LIKE'
    NOT_LIKE = 'NOT LIKE'
    IN = 'IN'
    GREATER_THAN = '>'
    LESS_THAN = '>'
    GREATER_THAN_OR_EQUAL_TO = '>='
    LESS_THAN_OR_EQUAL_TO = '>='
    POSIX = '~'


class DBAndOrEnum(Enum):
    AND = 'AND'
    OR = 'OR'


class DBConditionParamsTypeEnum(Enum):
    P = 0
    C = 1


__db_expression = {
    'field': {
        'concat': {
            'comma_lr': """CONCAT(',', %s, ',')"""
        },
        'cast': {
            'varchar': """CAST(%s AS VARCHAR)"""
        }
    },
    'params': {
        'like': {
            'lr': '%%%s%%',
            'comma_lr': '%%,%s,%%'
        },
        'in': ' (%s) '
    }
}

DBExpression = __db_expression


class DBConditionModel(object):

    field = ''
    field_expression = ''
    operator = DBOperatorEnum.EQUAL_TO
    params = None
    params_expression = ''
    params_type = DBConditionParamsTypeEnum.P
    params_brackets = False
    and_or = DBAndOrEnum.AND
    use_and_or = True
    inner_and_or = DBAndOrEnum.AND

    def __init__(self):
        pass

    def __init__(self, field='', field_expression='', operator=DBOperatorEnum.EQUAL_TO, params=None, params_expression='', params_type=DBConditionParamsTypeEnum.P, params_brackets=False, and_or=DBAndOrEnum.AND, use_and_or=True, inner_and_or=DBAndOrEnum.AND, use_inner_and_or=False):

        self.field = field
        self.field_expression = field_expression
        self.operator = operator
        # print('operator:', operator, type(operator), operator.value, type(operator.value))
        # print('self.operator:', self.operator, type(self.operator), self.operator.value, type(self.operator.value))
        self.params = params
        self.params_expression = params_expression
        self.params_type = params_type
        self.params_brackets = params_brackets
        self.and_or = and_or
        self.use_and_or = use_and_or
        self.use_inner_and_or = use_inner_and_or
        self.inner_and_or = inner_and_or


class DBConditionHelper(object):

    @staticmethod
    def get_in_condition(field, params, **kwargs):
        return DBConditionModel(
            field=field,
            params=params,
            params_brackets=True,
            operator=DBOperatorEnum.IN,
            **kwargs
        )

    @staticmethod
    def get_equal_condition(field, params):
        return DBConditionModel(
            field=field,
            params=params,
        )

    @staticmethod
    def get_like_lr_condition(field, params, has_comma=True, **kwargs):
        field_expression = ''
        params_expression = DBExpression['params']['like']['lr']
        if has_comma:
            field_expression = DBExpression['field']['concat']['comma_lr']
            params_expression = DBExpression['params']['like']['comma_lr']

        return DBConditionModel(
            field=field,
            field_expression=field_expression,
            operator=DBOperatorEnum.LIKE,
            params=params,
            params_expression=params_expression,
            **kwargs
        )

    @staticmethod
    def get_posix_condition(field, params, has_field_cast_varchar_exp=True, **kwargs):
        field_expression = ''
        if has_field_cast_varchar_exp:
            field_expression = DBExpression['field']['cast']['varchar']
        return DBConditionModel(
            field=field,
            field_expression=field_expression,
            operator=DBOperatorEnum.POSIX,
            params=params,
            **kwargs
        )


class PgDBHelper(BaseDBHelper):

    def connect(self):
        try:
            conn = psycopg2.connect(self.connect_string, cursor_factory=DictCursor)
            conn.set_client_encoding('UTF8')
            # print(conn)
            return conn
        except psycopg2.Error as e:
            print('psycopg2 error:', e.pgerror)
            return None

    def __analyze_condition(self, condition):
        __params = []
        __condition = ''
        __conditions = []

        operator = condition.operator.value
        params_type = condition.params_type

        use_and_or = condition.use_and_or
        and_or = condition.and_or.value
        if not use_and_or:
            and_or = ''

        use_inner_and_or = condition.use_inner_and_or
        inner_and_or = condition.inner_and_or.value

        # use_and_or = condition.use_and_or
        # and_or = condition.and_or.value
        # if not use_and_or:
        #     and_or = ''

        field = condition.field
        field_expression = condition.field_expression
        if field_expression != '':
            field = field_expression % field

        __c = ' ' + field + ' ' + operator + ' '
        # __c = (inner_and_or if use_inner_and_or else and_or) + ' ' + field + ' ' + operator + ' '
        # print('o:', o, type(o))
        # print('and_or:', and_or, type(and_or))
        # print('f:', f, type(f))
        # print('fe:', fe, type(fe))
        # print('__c:', __c)

        params = condition.params
        params_expression = condition.params_expression
        new_params = []

        if type(params) is not list:
            params = [params]

        # if pe != '':
            # p = pe % p
        new_params = [param if params_expression == '' else (params_expression % param) for param in params]

        # print('pe:', pe)
        # print('new_params:', new_params)

        for param in new_params:
            nc = __c
            if params_type == DBConditionParamsTypeEnum.P:
  
                if operator == 'IN':
                    param = tuple(param.split(','))
                    
                nc += ' %s ' # if not condition.params_brackets else ' (%s) '
                __params.append(param)
            else:
                nc += param

            # __condition += nc + ' '
            __conditions.append(nc)

        __condition = (inner_and_or if use_inner_and_or else and_or).join(__conditions)

        if use_inner_and_or:
            __condition = ' (' + __condition + ') '

        __condition = and_or + __condition

        # print('__params:', __params)
        # print('__condition:', __condition)

        return __params, __condition

    # def get_one(self, table_name, fields, sc=None, conditions=None):
    #     rst, cnt = self.get_list(table_name, fields, sc=sc, conditions=conditions, order_field=None, order_type=None, count=False, pagination=False)
    #     return rst[0] if len(rst) > 0 else None

    def get_list(self, table_name, fields, sc=None, conditions=None, rawConditions='', order_field='id', order_type='DESC', page_index=0, page_size=10, count=True, pagination=True):
        params = []
        condition = ''
        rst = None
        cnt = 0

        if conditions is not None:

            for c in conditions:
                __cc = ''
                template = ''
                t = type(c)

                if t is not list:
                    c = [c]
                    template = '%s'
                else:
                    template = ' AND (%s) '

                for c2 in c:
                    __params, __condition = self.__analyze_condition(c2)
                    params.extend(__params)
                    __cc += __condition
                # print('__cc:', __cc)

                condition += template % __cc
        
        if rawConditions != '':
            condition += rawConditions

        rst, cnt = self.get_list_base(table_name, fields, condition, params, order_field=order_field, order_type=order_type, sc=sc, page_index=page_index, page_size=page_size, count=count, pagination=pagination)
        return rst, cnt

    def get_list_base(self, table_name, fields, condition='', params=None, order_field='id', order_type='DESC', sc=None, page_index=0, page_size=10, count=True, pagination=True):
        cnt = 0
        if pagination:
            cnt_sql = """SELECT COUNT(1) AS cnt FROM """ + table_name + ' WHERE 1 = 1 ' + condition + """;"""
            rst = self.fetch_one(cnt_sql, params)
            cnt = rst['cnt']

        sql = """SELECT """ + fields + """ FROM """ + table_name
        sql += """ WHERE 1 = 1 """
        sql += condition

        if order_field is not None and order_type is not None:
            order_field = [order_field] if type(order_field) is str else order_field
            order_type = [order_type] if type(order_type) is str else order_type

            if len(order_type) != len(order_field):
                order_type = list(map(lambda x: order_type[0], range(len(order_field))))

            sort_list = list(map(lambda f, t: """ %s %s """ % (f, t), order_field, order_type))

            sql += """ ORDER BY """ + ','.join(sort_list)

        if pagination:
            if page_size > 0:
                sql += """ LIMIT %s OFFSET %s """
                params.append(page_size)
                params.append(page_size*page_index)

            # __page_size = 10
            # __page_index = 0

            # if sc is not None:
            #     __page_size = int(sc['pageSize'] if sc['pageSize'] is not None else 10)
            #     __page_index = int(sc['pageIndex'] if sc['pageIndex'] is not None else 1)-1
            # else:
            #     __page_size = page_size
            #     __page_index = page_index

            # if __page_size > 0:
            #     sql += """ LIMIT %s OFFSET %s """
            #     params.append(__page_size)
            #     params.append(__page_size*__page_index)

        sql += """;"""

        # print(sql, params)

        rst = self.fetch_all(sql, params)
        return rst, cnt

    def fetch_returning_id(self, sql, *params, **kw):
        sql = sql + """ RETURNING id;"""

        def callback(cur):
            rst = cur.fetchone()
            id = rst.get('id')
            return 0 if id is None else id

        return self.execute_sql(sql, callback, *params, **kw)

    def fetch_rowcount(self, sql, *params, **kw):
        def callback(cur):
            # print('cur:',cur)
            return cur.rowcount

        return self.execute_sql(sql, callback, *params, **kw)

    def fetch_one(self, sql, *params, **kw):
        def callback(cur):
            return cur.fetchone()

        return self.execute_sql(sql, callback, *params, **kw)

    def fetch_all(self, sql, *params, **kw):
        def callback(cur):
            return cur.fetchall()

        return self.execute_sql(sql, callback, *params, **kw)

    def execute_sql(self, sql, callback, *params, **kw):
        conn = self.connect()

        if conn == None:
            return None

        # print(sql, *params, type(params), len(params), **kw)
        is_multiple = False if kw.get('is_multiple') == None else kw['is_multiple']
        # print('--kw--:', kw, kw.get('is_multiple'), is_multiple)

        cur = conn.cursor(cursor_factory=RealDictCursor)
        # cur = conn.cursor()
        rst = None

        if is_multiple:
            psycopg2.extras.execute_values(cur, sql, params, page_size=9999)
        else:
            if params is not None:
                # print('sql:', cur.mogrify(sql, *params))
                cur.execute(sql, *params)
            else:
                cur.execute(sql)

        # rst = cur.fetchone()

        rst = callback(cur)
        conn.commit()
        conn.close()
        # print('rst:',rst)
        return rst

    def callproc(self, proc_name, *params):
        conn = self.connect()

        if conn == None:
            return None

        cur = conn.cursor()
        cur.callproc(proc_name, *params)
        rst = cur.fetchall()

        conn.commit()
        conn.close()
        return rst


"""
    # def execute_values(self, sql, *params):
    #     conn = db_helper.connect()

    #     if conn == None:
    #         return None

    #     print(params, type(params), list(params))

    #     cur = conn.cursor()
    #     psycopg2.extras.execute_values(cur, sql, params)
    #     rst = cur.rowcount
    #     conn.commit()
    #     conn.close()
    #     return rst
"""



# -*- coding=utf-8 -*-

import json

from Crypto.Cipher import AES
from Crypto.Hash import MD5

from binascii import b2a_hex, a2b_hex, hexlify, unhexlify
from base64 import b64encode, b64decode

from libs.src.pkcs7 import PKCS7Encoder

# __KEY = '12345678900000001234567890000000'
# __IV = '1234567890000000'

# _KEY = bytearray(__KEY, 'utf8')
# _IV = bytearray(__IV, 'utf8')


__KEY = '12345678900000001234567890000000'
__IV = '1234567890000000'
_KEY = bytes(__KEY, 'utf8')
_IV = bytes(__IV, 'utf8')


class CryptoHelper(object):

    KEY = _KEY
    IV = _IV

    # @property
    # def __KEY(self):
    #     return '12345678900000001234567890000000'

    # @property
    # def __IV(self):
    #     return '1234567890000000'

    # @property
    # def _KEY(self):
    #     return bytes(self.__KEY, 'utf8')

    # @property
    # def _IV(self):
    #     return bytes(self.__IV, 'utf8')

    @staticmethod
    def encrypt_by_md5(plain_text):
        h = MD5.new()
        h.update(bytes(plain_text, 'utf8'))
        return h.hexdigest()

    @staticmethod
    def encrypt_by_aes(plain_text):

        # key = self._KEY

        # key_length = len(key)
        # if (key_length >= 32):
        #     k = key[:32]
        # elif (key_length >= 24):
        #     k = key[:24]
        # else:
        #     k = key[:16]

        # print len(text)
        if type(plain_text) is not str:
            plain_text = json.dumps(plain_text)

        cryptor = AES.new(_KEY, AES.MODE_CBC, _IV)
        pad_text = PKCS7Encoder.encode(plain_text)
        # ciphertext = cryptor.encrypt(bytearray(pad_text,'utf-8'))
        ciphertext = cryptor.encrypt(pad_text)
        # print(ciphertext)
        return str(b2a_hex(ciphertext), encoding='utf-8').upper()
        # return ciphertext

    @staticmethod
    def decrypt_by_aes(ciphertext):
        cryptor = AES.new(_KEY, AES.MODE_CBC, _IV)
        pad_text = cryptor.decrypt(a2b_hex(ciphertext))
        # return str(plain_text).rstrip('\x0c')
        try:
            return PKCS7Encoder.decode(pad_text).decode('utf-8')
        except:
            return None



# -*- coding=utf-8 -*-

from helpers.src.utilities_helper import UtilitiesHelper


class ModelTransferService(object):
    # keys_relation = None
    # obj = None

    def __init__(self, obj):
        # self.keys_relation = []
        self.obj = obj

    # def transfer_to_py(self, data):
    #     # model = dict(map(lambda x: (x[1], '' if data.get(x[0]) is None else data[x[0]]), self.keys_relation))
    #     for key in self.keys_relation:
    #         ts_attr = key[0]
    #         d = data.get(ts_attr)
    #         setattr(self.obj, key[1], d)

    #     return self.obj

    def auto_transfer(self, data):
        keys = list(data.keys())
        for key in keys:
            snake_key = UtilitiesHelper.camel_2_snake(key)
            # print(key, snake_key)
            setattr(self.obj, snake_key, data.get(key))
        return self.obj



# -*- coding=utf-8 -*-

class BaseDBHelper(object):

    connect_string = ''
    cur = None

    def __init__(self, connect_string):
        self.connect_string = connect_string

    def connect(self):
        pass


    def fetch_returning_id(self, sql, *params, **kw):
        pass

    def fetch_rowcount(self, sql, *params, **kw):
        pass

    def fetch_one(self, sql, *params, **kw):
        pass

    def fetch_all(self, sql, *params, **kw):
        pass

    def execute_sql(self, sql, callback, *params, **kw):
        pass

    def callproc(self, proc_name, *params):
        pass



# -*- coding: utf-8 -*-

import json
import time
import random
import re
import datetime
import os
import string
from itsdangerous import TimedJSONWebSignatureSerializer as Serializer

from libs.src.typed import JSONObject


class UtilitiesHelper(object):
    def __init__(self):
        pass

    """[summary]
        json -> str
    Returns:
        [type] -- [description]
    """
    @staticmethod
    def json_dumps(data, ensure_ascii=True):
        return json.dumps(data, ensure_ascii=ensure_ascii)

    """[summary]
        json -> str
    Returns:
        [type] -- [description]
    """
    @staticmethod
    def json_dumps_dict(data, ensure_ascii=True):
        return json.dumps(data, default=lambda obj: UtilitiesHelper.json_default(obj), ensure_ascii=ensure_ascii)

    @staticmethod
    def json_default(value):
        if isinstance(value, datetime.datetime):
            return value.isoformat()
        else:
            return value.__dict__

    """[summary]
        str -> json
    Returns:
        [type] -- [description]
    """
    @staticmethod
    def json_loads_object_hook(data):
        return json.loads(data, object_hook=JSONObject)

    """[summary]
        str -> json
    Returns:
        [type] -- [description]
    """
    @staticmethod
    def json_loads(data):
        return json.loads(data)

    @staticmethod
    def decode_unicode(data):
        return data.replace('\n', '\\n').encode('utf-8').decode('unicode-escape')
        # return data.replace('\\"', '\\\\"').encode('utf-8').decode('unicode-escape').replace('\n', '\\n')
        # return data

    @staticmethod
    def tuple_to_dict(data, key_list):
        return dict(map(lambda x, y: (x, y), key_list, data))

    """[summary]

    Returns:
        yyyymmdd
    """
    @staticmethod
    def now_date():
        return UtilitiesHelper.now(f='%Y%m%d')

    """[summary]

    Returns:
        yyyy-mm-dd
    """
    @staticmethod
    def today():
        return datetime.date.today()

    """[summary]

    Returns:
        yyyy-mm-dd
    """
    @staticmethod
    def today_str():
        return str(UtilitiesHelper.today())

    @staticmethod
    def now_time():
        return UtilitiesHelper.now(f='%Y-%m-%d %H:%M:%S')

    @staticmethod
    def get_timestamp():
        return UtilitiesHelper.now('%Y%m%d%H%M%S')

    @staticmethod
    def now(f=None):
        if f is None:
            return time
        return time.strftime(f)

    @staticmethod
    def get_obj_property_names(obj):
        property_names = [p for p in dir(obj) if isinstance(getattr(obj, p), property)]
        return property_names

    @staticmethod
    def get_cls_fields_values(cls):
        fields = []
        values = []
        for prop in cls.__dict__:
            fields.append(prop)
            values.append(getattr(cls, prop))

        return fields, values

    @staticmethod
    def get_proj_root_path(root_path_name='backend'):
        current_path = os.path.abspath(os.path.dirname(__file__))
        root_path = current_path[:current_path.find(root_path_name)+len(root_path_name)]
        return root_path

    @staticmethod
    def sleep(t):
        time.sleep(t)

    @staticmethod
    def random(start=1000, stop=9999):
        return random.randrange(start, stop)

    @staticmethod
    def sub_non_numeric(text, numeric=True):
        number = re.sub('[^0-9]', '', text)
        if numeric:
            if number is not None and number != '':
                return int(number)
            return 0
        return number

    @staticmethod
    def filter_char(pattern, repl, text):
        if text is None:
            return ''
        return re.sub(pattern, repl, text)

    @staticmethod
    def check_url(path, prefix):
        return ('' if len(re.findall('http[s]?:\/\/', path)) > 0 else prefix) + path

    @staticmethod
    def get_char_list():
        # 1
        a = list(map(chr, range(ord('a'), ord('z') + 1)))
        # 2
        b = [chr(x) for x in range(ord('a'), ord('z') + 1)]
        # 3
        letter_list = string.ascii_letters
        number_list = string.digits
        numbers_letters = string.ascii_letters + string.digits

        return a

    @staticmethod
    def get_longest(arr, key=None):
        max_length, longest_element = max([(len((x if key is None else x.get(key))), x) for x in arr])
        return max_length, longest_element

    @staticmethod
    def get_length(char):
        try:
            row_l = len(char)
            utf8_l = len(char.encode('utf-8'))
            # print(char, row_l, utf8_l)
            return int((utf8_l - row_l) / 2 + row_l)
        except:
            return None
        return None

    @staticmethod
    def remove_list_empty(_list):
        while '' in _list:
            _list.remove('')
        return _list

    @staticmethod
    def camel_2_snake(value):
        s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', value)
        return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()

    @staticmethod
    def serializer(value, **kwargs):
        return Serializer(value, **kwargs)



# -*- coding=utf-8 -*-

from distutils.core import setup

from setuptools import setup, find_packages

setup(
    name="x_py_libs",
    version="0.0.7.3",
    # keywords = ("pip", "license","licensetool", "tool", "gm"),
    # description = "设备指纹获取、license生成、指纹与有效期验证工具",
    # long_description = "设备指纹获取、license生成、指纹与有效期验证工具",
    license="MIT Licence",

    # url = "https://github.com/gm19900510/licensetool",
    author="xiaxiazhu",
    author_email="xiaxiazhu147@gmail.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=[
        'psycopg2', 'pyodbc',
        'tornado',
        'Flask', 'Flask_Cors', 'Flask_RESTful',
        'redis',
        'six', 'pymediainfo', 'itsdangerous','Werkzeug',
        'requests',
        # 'pycrypto', 
        'pycryptodome'
    ]
)




# -*- coding=utf-8 -*-


import redis

from helpers.src.icache_helper import ICacheHelper


class RedisHelper(ICacheHelper):

    host = '127.0.0.1'
    port = 6379
    db = 0

    def __init__(self, host='127.0.0.1', port=6379, db=0, password=None):
        self.host = host
        self.port = port
        self.db = db
        self.password = password

        pool = redis.ConnectionPool(host=self.host, port=self.port, decode_responses=True)
        self.r = redis.Redis(connection_pool=pool, password=self.password)

    def set(self, key, value):
        self.r.set(key, value)

    def get(self, key):
        return self.r.get(key)

    def delete(self, key):
        self.r.delete(key)

    def hash_mset(self, key, value):
        self.r.hmset(key, value)

    def hash_get_all(self, key):
        return self.r.hgetall(key)



# -*- coding: utf-8 -*-

# 描述符基类,用于类型检查


class Typed(object):

    def __init__(self, name, expected_type):
        self.name = name
        self.expected_type = expected_type

    def __get__(self, instance, cls):
        if instance is None:
            return self
        else:
            return instance.__dict__[self.name]

    def __set__(self, instance, value):
        if not isinstance(value, self.expected_type):
            raise TypeError("Expected " + str(self.expected_type))
        instance.__dict__[self.name] = value

    def __delete__(self, instance):
        del instance.__dict__[self.name]

# 类装饰器，使用Typed类


def typeassert(**kwargs):
    def decorate(cls):
        for name, expected_type in kwargs.items():
            # 将名字name和期望类型创建一个Typed实例
            iTyped = Typed(name, expected_type)
            # 将名字name和iType，设置到cls中，作为对应cls.__dict__中的key，value
            setattr(cls, name, iTyped)
        return cls
    return decorate


class JSONObject:
    def __init__(self, d):
        self.__dict__ = d
        


# -*- coding=utf-8 -*-

from threading import Thread
from six.moves import queue as Queue

from helpers.src.io_helper import IOHelper
from helpers.src.requests_helper import RequestsHelper

class DownloadHelper(Thread):

    def __init__(self, queue, error_log_file_path, use_proxy=False, socks_proxy=False):
        Thread.__init__(self)
        self.queue = queue
        self.error_log_file_path = error_log_file_path
        self.use_proxy = use_proxy
        self.socks_proxy = socks_proxy

    def run(self):
        while True:
            url, save_path = self.queue.get()
            self.download(url, save_path)
            self.queue.task_done()

    def download(self, url, save_path):
        # print("Downloading %s from %s.\n" % (save_path, url))
        try:
            RequestsHelper.download(url, save_path=save_path, override=True, use_proxy=self.use_proxy, socks_proxy=self.socks_proxy)
        except Exception as err:
            text = '\rurl:\t\t\t\t' + url + '\rsave path:\t' + save_path + '\rerr:\t' + str(err) + '\r'
            IOHelper.save_to_file(self.error_log_file_path, text, mode='a+')